DOUBLE_HEIGHT   equ 1
TEXTYPOSMAX     equ 16

    MAC TWELVECHAR_LOGIC
twelvecharLogic
    lda measure
    cmp #SCROLLSTART
    bcs DoScroller
    lda looping
    bne DoScroller

#if charptr >= VARSTART
    lda #0
    ldx #23
ClearCharPtr
    sta charptr,X
    dex
    bpl ClearCharPtr
#endif

    rts

DoScroller
    lda beat
    asl
    and #$1F
    tax
    lda SinTab+32,X
    lsr
    lsr
    adc #4
    sta textypos

    lda frame
    and #7
    beq LoadNewChar
    rts
LoadNewChar

    ;huffman decoder
RestartDecoder
    lda #NSYM
    sta huffnsym
    lda #0
    sta REFP0       ;reset due to reflected Harry
    sta huffofs
    sta huffsym
NextBit
    dec nbits
    bpl BitsOK

    ;load next byte from stream - init/restart decoder if outside [Glyph0,FontEnd) and [Text,TextEnd)
    inc huffptr
    bne HuffPtrNoCarry
    inc huffptr+1
HuffPtrNoCarry

    lda huffptr+1
    cmp #>FontEnd
    bne NotOnFontEnd

    ldx huffptr
    cpx #<FontEnd
    bne NotOnFontEnd

    lda #<Text
    sta huffptr
    lda #>Text
    sta huffptr+1

NotOnFontEnd
    cmp #>Glyph0
    bcc HuffPtrInitial  ;initial bogus value. allows cheap init

    cmp #>TextEnd
    bcc HuffPtrOK
    lda huffptr
    cmp #<TextEnd
    bcc HuffPtrOK
HuffPtrInitial
    lda #<(Glyph0-1)
    sta huffptr
    lda #>(Glyph0-1)
    sta huffptr+1
    bne RestartDecoder  ;resync

HuffPtrOK
    lda #7
    ldx huffptr+1
    cpx #>Text
    bcs EightBits
    lda #1
EightBits
    sta nbits
    ldy #0
    lda (huffptr),Y
    sta bits
BitsOK
    ldx huffofs
    ldy huffnsym
    lda HuffTab,X
    sta huffnsym    ;if (bit == 1) huffnsym = HuffTab,X
    lsr bits
    bcc BitZero

    ;bit == 1
    inx
    stx huffofs
    bne FinishDecoding

BitZero
    txa
    adc huffnsym    ;HuffTab,X
    sta huffofs
    lda huffsym
    adc huffnsym    ;HuffTab,X
    sta huffsym
    tya
    sec
    sbc huffnsym    ;HuffTab,X
    sta huffnsym    ;if (bit == 0) huffnsym -= HuffTab,X

FinishDecoding
    ;A == huffnsym
    ;if 1 -> done
    cmp #1
    bne NextBit

    echo "Huffman decoder:",(*-LoadNewChar)d
    ;shift charptrs
    ldx #0
ShiftCharPtrs
    lda charptr+2,X
    sta charptr,X
    inx
    cpx #22
    bne ShiftCharPtrs

    ;huffsym = the decoded symbol. convert it to a pointer
#if FONTSZ > 1024
    echo "Font must be < 1 KiB"
    ERR
#endif
#if FONTSZ > 512
    ldx #0
#endif
    lda huffsym
    asl
    asl
#if FONTSZ > 512
    bcc SymOK
    inx
    inx
SymOK
#endif
    asl
    sta charptr+22
#if FONTSZ > 512
    txa
    adc #>Glyph0
#else
    lda #>Glyph0
    adc #0
#endif
    sta charptr+23

#if 1                   ;set to zero for pixel scrolling - it actually looks worse
TEXTX   equ 35
    lda #TEXTX
    ldx #0
    jsr PositionObject
    lda #TEXTX+16
#else
TEXTX   equ 32
    lda frame
    and #7
    eor #7
    clc
    adc #TEXTX
    ldx #0
    jsr PositionObject
    lda frame
    and #7
    eor #7
    clc
    adc #TEXTX+16
#endif
    inx
    stx VDELP0
    stx VDELP1
    jsr PositionObject

    rts
    ENDM

    MAC TWELVECHAR_KERNEL
twelvecharKernel
    lda #$50
    sta COLUBK
    lda #$06
    sta NUSIZ0
    sta NUSIZ1
    lda #$5E
    sta COLUP0
    sta COLUP1

    lda textypos
    clc
    adc #4
    tax
    jsr Burn

    ldx #%11111100
#if DOUBLE_HEIGHT
    lda #CHARH*2-1
    sta temp+1
#else
    ldy #CHARH-1
#endif
twelvecharKernelLoop
    lda #$80
    sta HMP0
    sta HMP1
    ;cy 66

#if DOUBLE_HEIGHT
    lda temp+1
#endif

    sta WSYNC
    sta HMOVE           ;right 8 pixels

#if DOUBLE_HEIGHT
    lsr
    tay
#else
    SLEEP 5
#endif

    lda (charptr+2),Y
    sax GRP0
    lda (charptr+6),Y
    sax GRP1
    lda (charptr+10),Y
    sax GRP0
    lda (charptr+14),Y
    sax GRP1
    lda (charptr+18),Y
    sax GRP0
    lda (charptr+22),Y
    sax GRP1
    sax GRP0

#if DOUBLE_HEIGHT
    SLEEP 9
#else
    SLEEP 8
#endif

    sta HMCLR   ;I found a video that explains how this works:
    sta HMOVE   ;http://www.youtube.com/watch?v=x0yQg8kHVcI

    SLEEP 6

    lda (charptr+0),Y
    sax GRP0
    lda (charptr+4),Y
    sax GRP1
    lda (charptr+8),Y
    sax GRP0
    lda (charptr+12),Y
    sax GRP1
    lda (charptr+16),Y
    sax GRP0
    lda (charptr+20),Y
    sax GRP1
    sax GRP0

#if DOUBLE_HEIGHT
    dec temp+1
#else
    dey
#endif
    bpl twelvecharKernelLoop
    CHECK_ALIGN twelvecharKernelLoop

    lda #TEXTYPOSMAX+1
    sec
    sbc textypos
    tax
twelveCharBurn
    sta WSYNC
    lda #0
    sta GRP0
    sta GRP1
    sta GRP0
    dex
    bne twelveCharBurn

    sta COLUBK
    sta WSYNC
    sta PF1
    sta PF2

    rts
    ENDM
